#!python
# coding=utf-8

import os
import netCDF4


def clone(src, dst_path, skip_globals, skip_dimensions, skip_variables):
    """
        Mostly ripped from nc3tonc4 in netCDF4-python.
        Added ability to skip dimension and variables.
        Removed all of the unpacking logic for shorts.
    """

    if os.path.exists(dst_path):
        os.unlink(dst_path)
    dst = netCDF4.Dataset(dst_path, 'w')

    # Global attributes
    for attname in src.ncattrs():
        if attname not in skip_globals:
            setattr(dst, attname, getattr(src, attname))

    # Dimensions
    unlimdim     = None
    unlimdimname = False
    for dimname, dim in src.dimensions.items():

        # Skip what we need to
        if dimname in skip_dimensions:
            continue

        if dim.isunlimited():
            unlimdim     = dim
            unlimdimname = dimname
            dst.createDimension(dimname, None)
        else:
            dst.createDimension(dimname, len(dim))

    # Variables
    for varname, ncvar in src.variables.items():

        # Skip what we need to
        if varname in skip_variables:
            continue

        hasunlimdim = False
        if unlimdimname and unlimdimname in ncvar.dimensions:
            hasunlimdim = True

        filler = None
        if hasattr(ncvar, '_FillValue'):
            filler = ncvar._FillValue

        if ncvar.chunking == "contiguous":
            var = dst.createVariable(varname, ncvar.dtype, ncvar.dimensions, fill_value=filler)
        else:
            var = dst.createVariable(varname, ncvar.dtype, ncvar.dimensions, fill_value=filler, chunksizes=ncvar.chunking())

        # Attributes
        for attname in ncvar.ncattrs():
            if attname == '_FillValue':
                continue
            else:
                setattr(var, attname, getattr(ncvar, attname))

        # Data
        nchunk = 1000
        if hasunlimdim:
            if nchunk:
                start = 0
                stop = len(unlimdim)
                step = nchunk
                if step < 1:
                    step = 1
                for n in range(start, stop, step):
                    nmax = n + nchunk
                    if nmax > len(unlimdim):
                        nmax = len(unlimdim)
                    idata = ncvar[n:nmax]
                    var[n:nmax] = idata
            else:
                idata = ncvar[:]
                var[0:len(unlimdim)] = idata
        else:
            idata = ncvar[:]
            var[:] = idata

        dst.sync()

    src.close()
    dst.close()



#!python
# coding=utf-8

import os
import random
import bisect
import calendar
from datetime import datetime

import netCDF4
import numpy as np
import pandas as pd

from pyaxiom import logger


class Profile(object):

    def __init__(self, df=None, global_attributes=None, variable_attributes=None, fill_value=None, vertical_positive=None, base_time=None):

        self.df = df if isinstance(df, pd.DataFrame) and not df.empty else pd.DataFrame()
        self.fill_value = fill_value or np.dtype('f8').type(-9999.9)
        self.global_attributes = global_attributes or {}
        self.variable_attributes = variable_attributes or {}
        self.vertical_positive = vertical_positive or 'down'
        self.base_time = base_time or 'seconds since 1970-01-01 00:00:00'

    @property
    def variable_attributes(self):
        defaults = {
            'time' : {
                'units' : self.base_time,
                'standard_name' : 'time',
                'long_name': 'time'
            },
            'latitude' : {
                'units' : 'degrees_north',
                'standard_name' : 'latitude',
                'long_name' : 'latitude',
                'axis': 'Y'
            },
            'longitude' : {
                'units' : 'degrees_east',
                'standard_name' : 'longitude',
                'long_name' : 'longitude',
                'axis': 'X'
            },
            'z' : {
                'units' : 'm',
                'standard_name' : 'depth',
                'long_name' : 'depth',
                'positive': self.vertical_positive,
                'axis': 'Z'
            },
            'profile' : {
                'cf_role' : 'profile_id'
            },
            'crs' : {
                'long_name' : 'http://www.opengis.net/def/crs/EPSG/0/4326',
                'grid_mapping_name' : 'latitude_longitude',
                'epsg_code' : 'EPSG:4326',
                'semi_major_axis' : float(6378137.0),
                'inverse_flattening' : float(298.257223563)
            },
            'platform' : {
                'definition' : "http://mmisw.org/ont/ioos/definition/stationID"
            }
        }

        defaults.update(self._variable_attributes)
        return defaults

    @variable_attributes.setter
    def variable_attributes(self, vas):
        self._variable_attributes = vas

    @property
    def global_attributes(self):
        gas = self._global_attributes
        gas.update({
           'geospatial_vertical_positive': self.vertical_positive,
           'date_created': datetime.utcnow().strftime("%Y-%m-%dT%H:%M:00Z"),
           'Conventions': 'CF-1.6',
           'Metadata_conventions': 'Unidata Dataset Discovery v1.0',
           'featureType': 'profile',
           'cdm_data_type': 'Profile'
        })

        if not self.df.empty:
            # Time
            starting = self.df['time'].min()
            ending = self.df['time'].max()
            duration = "P%sS" % str(int(round((ending - starting).total_seconds())))
            gas.update({
                'time_coverage_start': starting.strftime("%Y-%m-%dT%H:%M:00Z"),
                'time_coverage_end': ending.strftime("%Y-%m-%dT%H:%M:00Z"),
                'time_coverage_duration': duration,
            })
            diffs = self.df['time'].unique()[1:] - self.df['time'].unique()[:-1]
            uniqs, inverse = np.unique(diffs, return_inverse=True)
            if uniqs.size > 1:
                time_diffs = diffs[np.bincount(inverse).argmax()]
                gas.update({
                    'time_coverage_resolution': "P%sS" % str(round(time_diffs.astype('timedelta64[s]').astype(int)))
                })

            # Vertical
            gas.update({
                'geospatial_vertical_min': self.df['z'].min(),
                'geospatial_vertical_max': self.df['z'].max(),
            })

            # Horizontal
            gas.update({
                'geospatial_lat_min': self.df['latitude'].min(),
                'geospatial_lat_max': self.df['latitude'].max(),
                'geospatial_lon_min': self.df['longitude'].min(),
                'geospatial_lon_max': self.df['longitude'].max(),
            })

        return gas

    @global_attributes.setter
    def global_attributes(self, gas):
        # These are set by this script, we don't someone to be able to set them manually
        global_skips = ["time_coverage_start", "time_coverage_end", "time_coverage_duration", "time_coverage_resolution",
                        "featureType", "geospatial_vertical_positive", "geospatial_vertical_min", "geospatial_vertical_max",
                        "geospatial_lat_min", "geospatial_lon_min", "geospatial_lat_max", "geospatial_lon_max",
                        "Conventions", "date_created", "cdm_data_type"]

        for i in set(global_skips) & gas.keys():
            logger.warning("Ignoring global attribute {} because it is calculated or set automatically".format(i))

        self._global_attributes = { k: v for k, v in gas.items() if k not in global_skips }

    def export(self, output_file, file_type=None):
        # Make output directory
        if not os.path.exists(os.path.dirname(output_file)):
            os.makedirs(os.path.dirname(output_file))


class IncompleteProfile(Profile):

    def export(self, output_file):
        super(IncompleteProfile, self).export(output_file)

        with netCDF4.Dataset(output_file, 'w', clobber=True) as nc:

            gas = self.global_attributes
            nc.setncatts(gas)

            profiles = self.df.profile.unique().size
            profile_group = self.df.groupby('profile')
            max_z = profile_group.size().max()

            nc.createDimension('profile', profiles)
            nc.createDimension('z', max_z)

            profile = nc.createVariable('profile', self.df.profile.dtype, ('profile',))
            _, unique_profile_rows = np.unique(self.df.profile.values, return_index=True)
            profile[:] = list(range(profiles))

            time = nc.createVariable('time', int, ('profile',))
            time[:] = netCDF4.date2num([datetime.utcfromtimestamp(t) for t in self.df.time.unique().astype('<M8[s]').astype(int)], units=self.base_time)

            latitude = nc.createVariable('latitude', self.df.latitude.dtype, ('profile',))
            latitude[:] = self.df.latitude.values[unique_profile_rows]

            longitude = nc.createVariable('longitude', self.df.longitude.dtype, ('profile',))
            longitude[:] = self.df.longitude.values[unique_profile_rows]

            # Metadata variables
            nc.createVariable("crs", 'i4')
            nc.createVariable("platform", "i4")
            nc.setncattr('platform', 'platform')

            # Data vars
            reserved_columns = ['profile', 'time', 'latitude', 'longitude']
            for i, (name, p) in enumerate(profile_group):
                for c in [d for d in self.df.columns if d not in reserved_columns]:
                    var_name = c.split(' ')[0].lower()
                    fill = self.fill_value.astype(p[c].dtype)
                    if var_name not in nc.variables:
                        v = nc.createVariable(var_name, self.df[c].dtype, ('profile', 'z'), fill_value=fill)
                    else:
                        v = nc.variables[var_name]
                    assignable_values = p[c].fillna(fill).values
                    v[i, :len(assignable_values)] = assignable_values

            for k, v in self.variable_attributes.items():
                if k in nc.variables:
                    for n, z in v.items():
                        try:
                            nc.variables[k].setncattr(n, z)
                        except BaseException:
                            logger.warning('Could not set attribute {} on {}'.format(n, k))



#!python
# coding=utf-8

import random
import string
import itertools

from pyaxiom.urn import IoosUrn
from pyaxiom import logger


class DotDict(object):
    def __init__(self, *args, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __repr__(self):
        import pprint
        return pprint.pformat(vars(self), indent=2)


def dictify_urn(urn, combine_interval=True):
    """
        By default, this will put the `interval` as part of the `cell_methods`
        attribute (NetCDF CF style). To return `interval` as its own key, use
        the `combine_interval=False` parameter.
    """
    ioos_urn = IoosUrn.from_string(urn)

    if ioos_urn.valid() is False:
        return dict()

    if ioos_urn.asset_type != 'sensor':
        logger.error("This function only works on 'sensor' URNs.")
        return dict()

    if '#' in ioos_urn.component:
        standard_name, extras = ioos_urn.component.split('#')
    else:
        standard_name = ioos_urn.component
        extras = ''

    d = dict(standard_name=standard_name)

    # Discriminant
    if '-' in ioos_urn.component:
        d['discriminant'] = standard_name.split('-')[-1]
        d['standard_name'] = standard_name.split('-')[0]

    intervals = []
    cell_methods = []
    if extras:
        for section in extras.split(';'):
            key, values = section.split('=')
            if key == 'interval':
                # special case, intervals should be appended to the cell_methods
                for v in values.split(','):
                    intervals.append(v)
            else:
                if key == 'cell_methods':
                    value = [ x.replace('_', ' ').replace(':', ': ') for x in values.split(',') ]
                    cell_methods = value
                else:
                    value = ' '.join([x.replace('_', ' ').replace(':', ': ') for x in values.split(',')])
                    d[key] = value

    if combine_interval is True:
        if cell_methods and intervals:
            if len(cell_methods) == len(intervals):
                d['cell_methods'] = ' '.join([ '{} (interval: {})'.format(x[0], x[1].upper()) for x in zip(cell_methods, intervals) ])
            else:
                d['cell_methods'] = ' '.join(cell_methods)
                for i in intervals:
                    d['cell_methods'] += ' (interval: {})'.format(i.upper())
        elif cell_methods:
            d['cell_methods'] = ' '.join(cell_methods)
            for i in intervals:
                d['cell_methods'] += ' (interval: {})'.format(i.upper())
        elif intervals:
            raise ValueError("An interval without a cell_method is not allowed!  Not possible!")
    else:
        d['cell_methods'] = ' '.join(cell_methods)
        d['interval'] = ','.join(intervals).upper()

    if 'vertical_datum' in d:
        d['vertical_datum'] = d['vertical_datum'].upper()

    return d


def urnify(naming_authority, station_identifier, data):

    if isinstance(data, dict):
        return urnify_from_dict(naming_authority, station_identifier, data)
    else:
        d = dict(standard_name=getattr(data, 'standard_name', None),
                 bounds=getattr(data, 'bounds', None),
                 cell_methods=getattr(data, 'cell_methods', None),
                 vertical_datum=getattr(data, 'vertical_datum', None),
                 name=getattr(data, 'name', None),
                 discriminant=getattr(data, 'discriminant', None),
                 interval=getattr(data, 'interval', None))
        return urnify_from_dict(naming_authority, station_identifier, d)


def urnify_from_dict(naming_authority, station_identifier, data_dict):

    def clean_value(v):
        return v.replace('(', '').replace(')', '').strip().replace(' ', '_')
    extras = []
    intervals = []  # Because it can be part of cell_methods and its own dict key

    if 'cell_methods' in data_dict and data_dict['cell_methods']:
        cm = data_dict['cell_methods']
        keys = []
        values = []
        sofar = ''
        for i, c in enumerate(cm):
            if c == ":":
                if len(keys) == len(values):
                    keys.append(clean_value(sofar))
                else:
                    for j in reversed(range(0, i)):
                        if cm[j] == " ":
                            key = clean_value(cm[j+1:i])
                            values.append(clean_value(sofar.replace(key, '')))
                            keys.append(key)
                            break
                sofar = ''
            else:
                sofar += c
        # The last value needs appending
        values.append(clean_value(sofar))

        pairs = zip(keys, values)

        mems = []
        cell_intervals = []
        pairs = sorted(pairs)
        for group, members in itertools.groupby(pairs, lambda x: x[0]):
            if group == 'interval':
                cell_intervals = [m[1] for m in members]
            elif group in ['time', 'area']:  # Ignore 'comments'. May need to add more things here...
                member_strings = []
                for m in members:
                    member_strings.append('{}:{}'.format(group, m[1]))
                mems.append(','.join(member_strings))
        if mems:
            extras.append('cell_methods={}'.format(','.join(mems)))
        if cell_intervals:
            intervals += cell_intervals

    if 'bounds' in data_dict and data_dict['bounds']:
        extras.append('bounds={0}'.format(data_dict['bounds']))

    if 'vertical_datum' in data_dict and data_dict['vertical_datum']:
        extras.append('vertical_datum={0}'.format(data_dict['vertical_datum']))

    if 'interval' in data_dict and data_dict['interval']:
        if isinstance(data_dict['interval'], (list, tuple,)):
            intervals += data_dict['interval']
        elif isinstance(data_dict['interval'], str):
            intervals += [data_dict['interval']]

    if 'standard_name' in data_dict and data_dict['standard_name']:
        variable_name = data_dict['standard_name']
    elif 'name' in data_dict and data_dict['name']:
        variable_name = data_dict['name']
    else:
        variable_name = ''.join(random.choice(string.ascii_uppercase) for _ in range(8)).lower()
        logger.warning("Had to randomly generate a variable name: {0}".format(variable_name))

    if 'discriminant' in data_dict and data_dict['discriminant']:
        variable_name = '{}-{}'.format(variable_name, data_dict['discriminant'])

    if intervals:
        intervals = list(set(intervals))  # Unique them
        extras.append('interval={}'.format(','.join(intervals)))

    if extras:
        variable_name = '{0}#{1}'.format(variable_name, ';'.join(extras))

    u = IoosUrn(asset_type='sensor',
                authority=naming_authority,
                label=station_identifier,
                component=variable_name,
                version=None)

    return u.urn



import os

import unittest
import tempfile
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta

import pytest
import pytz

from pyaxiom.netcdf.grids import Collection

import logging
from pyaxiom import logger
logger.level = logging.INFO
logger.addHandler(logging.StreamHandler())


class NetcdfCollectionTestFromDirectory(unittest.TestCase):

    def setUp(self):
        input_folder = os.path.join(os.path.dirname(__file__), "resources/coamps/cencoos_4km/wnd_tru/10m/")
        self.c = Collection.from_directory(input_folder)

    def test_name(self):
        self.assertEqual(self.c.aggregation.name, "U.S. Navy Fleet Numerical Meteorology and Oceanography Center Forecast/Uninitialized Analysis/Image Product")

    def test_members(self):
        self.assertEqual(len(self.c.aggregation.members), 14)

    def test_time(self):
        self.assertEqual(self.c.aggregation.starting, datetime(2014, 6, 20, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(self.c.aggregation.ending, datetime(2014, 7, 19, 23, 0, tzinfo=pytz.utc))


class NetcdfCollectionTestFromDirectoryNoNcmlToMembers(unittest.TestCase):

    def setUp(self):
        input_folder = os.path.join(os.path.dirname(__file__), "resources/coamps/cencoos_4km/wnd_tru/10m/")
        self.c = Collection.from_directory(input_folder, apply_to_members=False)

    def test_name(self):
        self.assertEqual(self.c.aggregation.name, "U.S. Navy Fleet Numerical Meteorology and Oceanography Center Forecast/Uninitialized Analysis/Image Product")

    def test_members(self):
        self.assertEqual(len(self.c.aggregation.members), 14)

    def test_time(self):
        self.assertEqual(self.c.aggregation.starting, datetime(2014, 6, 20, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(self.c.aggregation.ending, datetime(2014, 7, 19, 23, 0, tzinfo=pytz.utc))


class NetcdfCollectionTestFromGlob(unittest.TestCase):

    def setUp(self):
        glob_string = os.path.join(os.path.dirname(__file__), "resources/coamps/cencoos_4km/wnd_tru/10m/*.nc")
        self.c = Collection.from_glob(glob_string)

    def test_name(self):
        self.assertEqual(self.c.aggregation.name, "U.S. Navy Fleet Numerical Meteorology and Oceanography Center Forecast/Uninitialized Analysis/Image Product")

    def test_members(self):
        self.assertEqual(len(self.c.aggregation.members), 14)

    def test_time(self):
        self.assertEqual(self.c.aggregation.starting, datetime(2014, 6, 20, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(self.c.aggregation.ending, datetime(2014, 7, 19, 23, 0, tzinfo=pytz.utc))


class NetcdfCollectionTestFromNestedGlobAndNcml(unittest.TestCase):

    def setUp(self):
        glob_string = os.path.join(os.path.dirname(__file__), "resources/coamps/cencoos_4km/wnd_tru/**/*.nc")
        ncml        = """<?xml version="1.0" encoding="UTF-8"?>
        <netcdf xmlns="http://www.unidata.ucar.edu/namespaces/netcdf/ncml-2.2">
            <attribute name="title" value="changed" />
        </netcdf>
        """
        self.c = Collection.from_glob(glob_string, ncml=ncml)

    def test_name(self):
        self.assertEqual(self.c.aggregation.name, "changed")

    def test_members(self):
        self.assertEqual(len(self.c.aggregation.members), 14)

    def test_time(self):
        self.assertEqual(self.c.aggregation.starting, datetime(2014, 6, 20, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(self.c.aggregation.ending, datetime(2014, 7, 19, 23, 0, tzinfo=pytz.utc))


class NetcdfCollectionTestFromNcml(unittest.TestCase):

    def setUp(self):
        input_ncml = os.path.join(os.path.dirname(__file__), "resources/coamps_10km_wind.ncml")
        self.c = Collection.from_ncml_file(input_ncml)

    def test_name(self):
        self.assertEqual(self.c.aggregation.name, "U.S. Navy Fleet Numerical Meteorology and Oceanography Center Forecast/Uninitialized Analysis/Image Product")

    def test_members(self):
        self.assertEqual(len(self.c.aggregation.members), 14)

    def test_time(self):
        self.assertEqual(self.c.aggregation.starting, datetime(2014, 6, 20, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(self.c.aggregation.ending, datetime(2014, 7, 19, 23, 0, tzinfo=pytz.utc))

    def test_yearly_bins(self):
        starting = self.c.aggregation.starting.replace(microsecond=0, second=0, minute=0, hour=0, day=1, month=1)
        bins = self.c.bins(delta=relativedelta(years=+1), starting=starting)
        self.assertEqual(len(bins), 1)

        first_month = bins[0]
        self.assertEqual(first_month.starting, datetime(2014, 1, 1, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(first_month.ending, datetime(2015, 1, 1, 0, 0, tzinfo=pytz.utc))

    def test_bimonthy_bins(self):
        starting = self.c.aggregation.starting.replace(microsecond=0, second=0, minute=0, hour=0, day=1)
        bins = self.c.bins(delta=relativedelta(months=+2), starting=starting)
        self.assertEqual(len(bins), 1)

        first_month = bins[0]
        self.assertEqual(first_month.starting, datetime(2014, 6, 1, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(first_month.ending, datetime(2014, 8, 1, 0, 0, tzinfo=pytz.utc))

    def test_monthly_bins(self):
        starting = self.c.aggregation.starting.replace(microsecond=0, second=0, minute=0, hour=0, day=1)
        bins = self.c.bins(delta=relativedelta(months=+1), starting=starting)
        self.assertEqual(len(bins), 2)

        first_month = bins[0]
        self.assertEqual(first_month.starting, datetime(2014, 6, 1, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(first_month.ending, datetime(2014, 7, 1, 0, 0, tzinfo=pytz.utc))

        second_month = bins[1]
        self.assertEqual(second_month.starting, datetime(2014, 7, 1, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(second_month.ending, datetime(2014, 8, 1, 0, 0, tzinfo=pytz.utc))

    def test_daily_bins(self):
        starting = self.c.aggregation.starting.replace(microsecond=0, second=0, minute=0, hour=0)
        bins = self.c.bins(delta=relativedelta(days=+1), starting=starting)
        self.assertEqual(len(bins), 2)

        first_day = bins[0]
        self.assertEqual(first_day.starting, datetime(2014, 6, 20, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(first_day.ending, datetime(2014, 6, 21, 0, 0, tzinfo=pytz.utc))

        second_day = bins[1]
        self.assertEqual(second_day.starting, datetime(2014, 7, 19, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(second_day.ending, datetime(2014, 7, 20, 0, 0, tzinfo=pytz.utc))

    def test_hard_start(self):
        starting = self.c.aggregation.starting.replace(microsecond=0, second=0, minute=0, hour=0, day=1)
        hard_start = datetime(2014, 7, 1, 0, 0, tzinfo=pytz.utc)
        bins = self.c.bins(delta=relativedelta(months=+1), starting=starting, hard_start=hard_start)
        self.assertEqual(len(bins), 1)

        second_month = bins[0]
        self.assertEqual(len(second_month.members), 4)
        self.assertEqual(second_month.starting, datetime(2014, 7, 1, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(second_month.ending, datetime(2014, 8, 1, 0, 0, tzinfo=pytz.utc))

    def test_hard_end(self):
        starting = self.c.aggregation.starting.replace(microsecond=0, second=0, minute=0, hour=0, day=1)
        hard_end = datetime(2014, 7, 1, 0, 0, tzinfo=pytz.utc)
        bins = self.c.bins(delta=relativedelta(months=+1), starting=starting, hard_end=hard_end)
        self.assertEqual(len(bins), 1)

        first_month = bins[0]
        self.assertEqual(len(first_month.members), 10)
        self.assertEqual(first_month.starting, datetime(2014, 6, 1, 0, 0, tzinfo=pytz.utc))
        self.assertEqual(first_month.ending, datetime(2014, 7, 1, 0, 0, tzinfo=pytz.utc))

    @pytest.mark.skipif(os.environ.get("TRAVIS_PYTHON_VERSION") is not None,
                        reason="No workie in Travis")
    def test_combine(self):
        output_file = tempfile.NamedTemporaryFile().name
        members = [ m.path for m in self.c.aggregation.members ]
        Collection.combine(members=members, output_file=output_file)
        self.assertTrue(os.path.isfile(output_file))
        os.remove(output_file)

    @pytest.mark.skipif(os.environ.get("TRAVIS_PYTHON_VERSION") is not None,
                        reason="No workie in Travis")
    def test_combine_passing_members(self):
        output_file = tempfile.NamedTemporaryFile().name
        Collection.combine(members=self.c.aggregation.members, output_file=output_file)
        self.assertTrue(os.path.isfile(output_file))
        os.remove(output_file)

    @pytest.mark.skipif(os.environ.get("TRAVIS_PYTHON_VERSION") is not None,
                        reason="No workie in Travis")
    def test_combine_with_dimension(self):
        output_file = tempfile.NamedTemporaryFile().name
        members = [ m.path for m in self.c.aggregation.members ]
        Collection.combine(members=members, output_file=output_file, dimension='time')
        self.assertTrue(os.path.isfile(output_file))
        os.remove(output_file)

    @pytest.mark.skipif(os.environ.get("TRAVIS_PYTHON_VERSION") is not None,
                        reason="No workie in Travis")
    def test_combine_with_dimension_and_stride(self):
        output_file = tempfile.NamedTemporaryFile().name
        members = [ m.path for m in self.c.aggregation.members ]
        Collection.combine(members=members, output_file=output_file, dimension='time', start_index=1, stop_index=6, stride=2)
        self.assertTrue(os.path.isfile(output_file))
        os.remove(output_file)
